<?php
/*******************************************************************************
*    myMVC - Model-View-Controller for PHP                                     *
*    Copyright (C) 2012 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU Lesser General Public License as published  *
*    by the Free Software Foundation; version 2.1 of the License.              *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU Lesser General Public License for more details.                       *
*                                                                              *
*    You should have received a copy of the GNU Lesser General Public License  *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/
(include_once('Mail.php')) or die("PEAR-Package 'Mail' is not installed");
(include_once('Mail/mime.php')) or die("PEAR-Package 'Mail/mime' is not installed");

/**
 * Generic error handler.
 * It supports displaying on browser and sending
 * E-Mail to a configured address (see $config['ERROR']['ALERT'] in mymvc.php)
 */
class myMVC_ErrorHandler
{
  /**
   * Display an error on the browser 
   * 
   * @param Exception $exception
   */   
  public static function systemError($exception)
  {
    global $config;

    myMVC_ErrorHandler::sendAlert($exception->getMessage()."\n".$exception->getTraceAsString());
    if($config['DISPLAY_SYSERROR'])
    {
      myMVC_Logger::doLog(__METHOD__, __LINE__, 'sysError: redirected to '.PAGE_ERROR, ALERT_WARNING, false);
      if(headers_sent())
        print "<META http-equiv=\"refresh\" content=\"0;url=".PAGE_ERROR."\">";
      else
        header('Location:'.PAGE_ERROR);
    }
    exit;
  }

  /**
   * sends an alert to the configured E-Mail or Nagios server
   * 
   * @param  string  $msg       Text to be sent
   * @param  int     $type      Alert type (see mymvc.php). Default ALERT_CRITICAL
   */
  public static function sendAlert($msg, $type = ALERT_CRITICAL)
  {
    global $config;

    list($aType, $address) = explode(':', $config['ERROR']['ALERT']);
    switch($aType)
    {
      case 'mailto':
        $hdrs = array(
                'From' => $config['MAIL']['FROM'],
                'To' => $address,
                'Subject' => sprintf("myMVC-Error on %s (%s)", $config['nodeName'], myMVC_Logger::$levelString[$type]),
                'Message-Id' => sprintf("<%s.%d@%s>", date('YmdHis'), $config['myPID'], $config['nodeName']),
                'Date' => date('r')
                );
        $toSendRecipients = array($address);
        $mime = new Mail_mime("\n");
        $mime->setTXTBody($msg);
        $body = $mime->get(array(
                                "text_charset"    => "UTF-8",
                                "head_charset"    => "UTF-8"));
        $hdrs = $mime->headers($hdrs);
        $mail =& Mail::factory('smtp',
                                array('host' => $config['MAIL']['SMTP'],
                                      'auth' => $config['MAIL']['AUTH'],
                                      'username' => $config['MAIL']['USER'],
                                      'password' => $config['MAIL']['PASS'],
                                      'localhost' => $config['nodeName']));
        $mail->send($toSendRecipients, $hdrs, $body);
      break;  
      case 'nsca':
        $fp = popen($address, 'w');
        $msg = str_replace("\n", ' - ', $msg);
        $nodeName = $config['nodeName'];
        fwrite($fp, "php\t$nodeName\t$type\t$msg\n");
        pclose($fp);
      break;  
      default:
        myMVC_Logger::doLog(__METHOD__, __LINE__, "ErrorHandler: sendAlert: unknown type $aType: check configuration file", $type, false);
      break;        
    }
  }

  /**
   * Set the default catcher for uncatched exceptions
   */
  public static function setExceptionCatcher()
  {
    set_exception_handler(array('myMVC_ErrorHandler', 'systemError'));
  }

  /**
   * Set a user defined function (or class-method)
   * for uncatched exception
   * 
   * @param string $catcher     Function or method name.
   *                            If method, then in the format class::method 
   */
  public static function setUserExceptionCatcher($catcher)
  {
    if(strpos($catcher, '::') !== false)
    {
      list($class, $method) = explode('::', $catcher);
      if(!class_exists($class))
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Class %s is not defined", $class));
        throw new myMVC_MVCException($msg);
      }
      if(!is_callable(array($class, $method)))
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Class %s has no method %s", $class, $method));
        throw new myMVC_MVCException($msg);
      }
      set_exception_handler(array($class, $method));
    }
    else
    {
      if(!is_callable($catcher))
      {
        $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, sprintf("Function %s does not exists", $catcher));
        throw new myMVC_MVCException($msg);
      }
      set_exception_handler($catcher);
    }
  }
}
?>
